@@ -11,10 +11,11 @@
 /// \brief Defines facilities for reading and writing on-disk hash tables.
 ///
 //===----------------------------------------------------------------------===//
-#ifndef LLVM_SUPPORT_ONDISKHASHTABLE_H
-#define LLVM_SUPPORT_ONDISKHASHTABLE_H
+#ifndef LLVM_SUPPORT_ON_DISK_HASH_TABLE_H
+#define LLVM_SUPPORT_ON_DISK_HASH_TABLE_H
 
 #include "llvm/Support/Allocator.h"
+#include "llvm/Support/AlignOf.h"
 #include "llvm/Support/DataTypes.h"
 #include "llvm/Support/EndianStream.h"
 #include "llvm/Support/Host.h"
@@ -52,8 +53,6 @@ namespace llvm {
 ///   /// Write Data to Out.  DataLen is the length from EmitKeyDataLength.
 ///   static void EmitData(raw_ostream &Out, key_type_ref Key,
 ///                        data_type_ref Data, offset_type DataLen);
-///   /// Determine if two keys are equal. Optional, only needed by contains.
-///   static bool EqualKey(key_type_ref Key1, key_type_ref Key2);
 /// };
 /// \endcode
 template <typename Info> class OnDiskChainedHashTableGenerator {
@@ -76,10 +75,13 @@ template <typename Info> class OnDiskChainedHashTableGenerator {
   llvm::SpecificBumpPtrAllocator<Item> BA;
 
   /// \brief A linked list of values in a particular hash bucket.
-  struct Bucket {
+  class Bucket {
+  public:
     offset_type Off;
-    unsigned Length;
     Item *Head;
+    unsigned Length;
+
+    Bucket() {}
   };
 
   Bucket *Buckets;
@@ -123,21 +125,13 @@ public:
   /// Uses the provided Info instead of a stack allocated one.
   void insert(typename Info::key_type_ref Key,
               typename Info::data_type_ref Data, Info &InfoObj) {
+
     ++NumEntries;
     if (4 * NumEntries >= 3 * NumBuckets)
       resize(NumBuckets * 2);
     insert(Buckets, NumBuckets, new (BA.Allocate()) Item(Key, Data, InfoObj));
   }
 
-  /// \brief Determine whether an entry has been inserted.
-  bool contains(typename Info::key_type_ref Key, Info &InfoObj) {
-    unsigned Hash = InfoObj.ComputeHash(Key);
-    for (Item *I = Buckets[Hash & (NumBuckets - 1)].Head; I; I = I->Next)
-      if (I->Hash == Hash && InfoObj.EqualKey(I->Key, Key))
-        return true;
-    return false;
-  }
-
   /// \brief Emit the table to Out, which must not be at offset 0.
   offset_type Emit(raw_ostream &Out) {
     Info InfoObj;
@@ -151,22 +145,6 @@ public:
     using namespace llvm::support;
     endian::Writer<little> LE(Out);
 
-    // Now we're done adding entries, resize the bucket list if it's
-    // significantly too large. (This only happens if the number of
-    // entries is small and we're within our initial allocation of
-    // 64 buckets.) We aim for an occupancy ratio in [3/8, 3/4).
-    //
-    // As a special case, if there are two or fewer entries, just
-    // form a single bucket. A linear scan is fine in that case, and
-    // this is very common in C++ class lookup tables. This also
-    // guarantees we produce at least one bucket for an empty table.
-    //
-    // FIXME: Try computing a perfect hash function at this point.
-    unsigned TargetNumBuckets =
-        NumEntries <= 2 ? 1 : NextPowerOf2(NumEntries * 4 / 3);
-    if (TargetNumBuckets != NumBuckets)
-      resize(TargetNumBuckets);
-
     // Emit the payload of the table.
     for (offset_type I = 0; I < NumBuckets; ++I) {
       Bucket &B = Buckets[I];
@@ -186,28 +164,14 @@ public:
         LE.write<typename Info::hash_value_type>(I->Hash);
         const std::pair<offset_type, offset_type> &Len =
             InfoObj.EmitKeyDataLength(Out, I->Key, I->Data);
-#ifdef NDEBUG
         InfoObj.EmitKey(Out, I->Key, Len.first);
         InfoObj.EmitData(Out, I->Key, I->Data, Len.second);
-#else
-        // In asserts mode, check that the users length matches the data they
-        // wrote.
-        uint64_t KeyStart = Out.tell();
-        InfoObj.EmitKey(Out, I->Key, Len.first);
-        uint64_t DataStart = Out.tell();
-        InfoObj.EmitData(Out, I->Key, I->Data, Len.second);
-        uint64_t End = Out.tell();
-        assert(offset_type(DataStart - KeyStart) == Len.first &&
-               "key length does not match bytes written");
-        assert(offset_type(End - DataStart) == Len.second &&
-               "data length does not match bytes written");
-#endif
       }
     }
 
     // Pad with zeros so that we can start the hashtable at an aligned address.
     offset_type TableOff = Out.tell();
-    uint64_t N = llvm::OffsetToAlignment(TableOff, alignof(offset_type));
+    uint64_t N = llvm::OffsetToAlignment(TableOff, alignOf<offset_type>());
     TableOff += N;
     while (N--)
       LE.write<uint8_t>(0);
@@ -278,12 +242,11 @@ template <typename Info> class OnDiskChainedHashTable {
   Info InfoObj;
 
 public:
-  typedef Info InfoType;
   typedef typename Info::internal_key_type internal_key_type;
   typedef typename Info::external_key_type external_key_type;
-  typedef typename Info::data_type data_type;
-  typedef typename Info::hash_value_type hash_value_type;
-  typedef typename Info::offset_type offset_type;
+  typedef typename Info::data_type         data_type;
+  typedef typename Info::hash_value_type   hash_value_type;
+  typedef typename Info::offset_type       offset_type;
 
   OnDiskChainedHashTable(offset_type NumBuckets, offset_type NumEntries,
                          const unsigned char *Buckets,
@@ -295,21 +258,6 @@ public:
            "'buckets' must have a 4-byte alignment");
   }
 
-  /// Read the number of buckets and the number of entries from a hash table
-  /// produced by OnDiskHashTableGenerator::Emit, and advance the Buckets
-  /// pointer past them.
-  static std::pair<offset_type, offset_type>
-  readNumBucketsAndEntries(const unsigned char *&Buckets) {
-    assert((reinterpret_cast<uintptr_t>(Buckets) & 0x3) == 0 &&
-           "buckets should be 4-byte aligned.");
-    using namespace llvm::support;
-    offset_type NumBuckets =
-        endian::readNext<offset_type, little, aligned>(Buckets);
-    offset_type NumEntries =
-        endian::readNext<offset_type, little, aligned>(Buckets);
-    return std::make_pair(NumBuckets, NumEntries);
-  }
-
   offset_type getNumBuckets() const { return NumBuckets; }
   offset_type getNumEntries() const { return NumEntries; }
   const unsigned char *getBase() const { return Base; }
@@ -324,35 +272,25 @@ public:
     Info *InfoObj;
 
   public:
-    iterator() : Key(), Data(nullptr), Len(0), InfoObj(nullptr) {}
+    iterator() : Data(nullptr), Len(0) {}
     iterator(const internal_key_type K, const unsigned char *D, offset_type L,
              Info *InfoObj)
         : Key(K), Data(D), Len(L), InfoObj(InfoObj) {}
 
     data_type operator*() const { return InfoObj->ReadData(Key, Data, Len); }
-
-    const unsigned char *getDataPtr() const { return Data; }
-    offset_type getDataLen() const { return Len; }
-
     bool operator==(const iterator &X) const { return X.Data == Data; }
     bool operator!=(const iterator &X) const { return X.Data != Data; }
   };
 
   /// \brief Look up the stored data for a particular key.
-  iterator find(const external_key_type &EKey, Info *InfoPtr = nullptr) {
-    const internal_key_type &IKey = InfoObj.GetInternalKey(EKey);
-    hash_value_type KeyHash = InfoObj.ComputeHash(IKey);
-    return find_hashed(IKey, KeyHash, InfoPtr);
-  }
-
-  /// \brief Look up the stored data for a particular key with a known hash.
-  iterator find_hashed(const internal_key_type &IKey, hash_value_type KeyHash,
-                       Info *InfoPtr = nullptr) {
-    using namespace llvm::support;
-
+  iterator find(const external_key_type &EKey, Info *InfoPtr = 0) {
     if (!InfoPtr)
       InfoPtr = &InfoObj;
 
+    using namespace llvm::support;
+    const internal_key_type &IKey = InfoObj.GetInternalKey(EKey);
+    hash_value_type KeyHash = InfoObj.ComputeHash(IKey);
+
     // Each bucket is just an offset into the hash table file.
     offset_type Idx = KeyHash & (NumBuckets - 1);
     const unsigned char *Bucket = Buckets + sizeof(offset_type) * Idx;
@@ -415,11 +353,17 @@ public:
   static OnDiskChainedHashTable *Create(const unsigned char *Buckets,
                                         const unsigned char *const Base,
                                         const Info &InfoObj = Info()) {
+    using namespace llvm::support;
     assert(Buckets > Base);
-    auto NumBucketsAndEntries = readNumBucketsAndEntries(Buckets);
-    return new OnDiskChainedHashTable<Info>(NumBucketsAndEntries.first,
-                                            NumBucketsAndEntries.second,
-                                            Buckets, Base, InfoObj);
+    assert((reinterpret_cast<uintptr_t>(Buckets) & 0x3) == 0 &&
+           "buckets should be 4-byte aligned.");
+
+    offset_type NumBuckets =
+        endian::readNext<offset_type, little, aligned>(Buckets);
+    offset_type NumEntries =
+        endian::readNext<offset_type, little, aligned>(Buckets);
+    return new OnDiskChainedHashTable<Info>(NumBuckets, NumEntries, Buckets,
+                                            Base, InfoObj);
   }
 };
 
@@ -438,30 +382,40 @@ public:
   typedef typename base_type::hash_value_type   hash_value_type;
   typedef typename base_type::offset_type       offset_type;
 
-private:
+  OnDiskIterableChainedHashTable(offset_type NumBuckets, offset_type NumEntries,
+                                 const unsigned char *Buckets,
+                                 const unsigned char *Payload,
+                                 const unsigned char *Base,
+                                 const Info &InfoObj = Info())
+      : base_type(NumBuckets, NumEntries, Buckets, Base, InfoObj),
+        Payload(Payload) {}
+
   /// \brief Iterates over all of the keys in the table.
-  class iterator_base {
+  class key_iterator {
     const unsigned char *Ptr;
     offset_type NumItemsInBucketLeft;
     offset_type NumEntriesLeft;
+    Info *InfoObj;
 
   public:
     typedef external_key_type value_type;
 
-    iterator_base(const unsigned char *const Ptr, offset_type NumEntries)
-        : Ptr(Ptr), NumItemsInBucketLeft(0), NumEntriesLeft(NumEntries) {}
-    iterator_base()
-        : Ptr(nullptr), NumItemsInBucketLeft(0), NumEntriesLeft(0) {}
+    key_iterator(const unsigned char *const Ptr, offset_type NumEntries,
+                 Info *InfoObj)
+        : Ptr(Ptr), NumItemsInBucketLeft(0), NumEntriesLeft(NumEntries),
+          InfoObj(InfoObj) {}
+    key_iterator()
+        : Ptr(nullptr), NumItemsInBucketLeft(0), NumEntriesLeft(0),
+          InfoObj(0) {}
 
-    friend bool operator==(const iterator_base &X, const iterator_base &Y) {
+    friend bool operator==(const key_iterator &X, const key_iterator &Y) {
       return X.NumEntriesLeft == Y.NumEntriesLeft;
     }
-    friend bool operator!=(const iterator_base &X, const iterator_base &Y) {
+    friend bool operator!=(const key_iterator &X, const key_iterator &Y) {
       return X.NumEntriesLeft != Y.NumEntriesLeft;
     }
 
-    /// Move to the next item.
-    void advance() {
+    key_iterator &operator++() { // Preincrement
       using namespace llvm::support;
       if (!NumItemsInBucketLeft) {
         // 'Items' starts with a 16-bit unsigned integer representing the
@@ -478,58 +432,25 @@ private:
       --NumItemsInBucketLeft;
       assert(NumEntriesLeft);
       --NumEntriesLeft;
-    }
-
-    /// Get the start of the item as written by the trait (after the hash and
-    /// immediately before the key and value length).
-    const unsigned char *getItem() const {
-      return Ptr + (NumItemsInBucketLeft ? 0 : 2) + sizeof(hash_value_type);
-    }
-  };
-
-public:
-  OnDiskIterableChainedHashTable(offset_type NumBuckets, offset_type NumEntries,
-                                 const unsigned char *Buckets,
-                                 const unsigned char *Payload,
-                                 const unsigned char *Base,
-                                 const Info &InfoObj = Info())
-      : base_type(NumBuckets, NumEntries, Buckets, Base, InfoObj),
-        Payload(Payload) {}
-
-  /// \brief Iterates over all of the keys in the table.
-  class key_iterator : public iterator_base {
-    Info *InfoObj;
-
-  public:
-    typedef external_key_type value_type;
-
-    key_iterator(const unsigned char *const Ptr, offset_type NumEntries,
-                 Info *InfoObj)
-        : iterator_base(Ptr, NumEntries), InfoObj(InfoObj) {}
-    key_iterator() : iterator_base(), InfoObj() {}
-
-    key_iterator &operator++() {
-      this->advance();
       return *this;
     }
     key_iterator operator++(int) { // Postincrement
-      key_iterator tmp = *this;
-      ++*this;
-      return tmp;
-    }
-
-    internal_key_type getInternalKey() const {
-      auto *LocalPtr = this->getItem();
-
-      // Determine the length of the key and the data.
-      auto L = Info::ReadKeyDataLength(LocalPtr);
-
-      // Read the key.
-      return InfoObj->ReadKey(LocalPtr, L.first);
+      key_iterator tmp = *this; ++*this; return tmp;
     }
 
     value_type operator*() const {
-      return InfoObj->GetExternalKey(getInternalKey());
+      const unsigned char *LocalPtr = Ptr;
+      if (!NumItemsInBucketLeft)
+        LocalPtr += 2; // number of items in bucket
+      LocalPtr += sizeof(hash_value_type); // Skip the hash.
+
+      // Determine the length of the key and the data.
+      const std::pair<offset_type, offset_type> &L =
+          Info::ReadKeyDataLength(LocalPtr);
+
+      // Read the key.
+      const internal_key_type &Key = InfoObj->ReadKey(LocalPtr, L.first);
+      return InfoObj->GetExternalKey(Key);
     }
   };
 
@@ -543,7 +464,10 @@ public:
   }
 
   /// \brief Iterates over all the entries in the table, returning the data.
-  class data_iterator : public iterator_base {
+  class data_iterator {
+    const unsigned char *Ptr;
+    offset_type NumItemsInBucketLeft;
+    offset_type NumEntriesLeft;
     Info *InfoObj;
 
   public:
@@ -551,24 +475,51 @@ public:
 
     data_iterator(const unsigned char *const Ptr, offset_type NumEntries,
                   Info *InfoObj)
-        : iterator_base(Ptr, NumEntries), InfoObj(InfoObj) {}
-    data_iterator() : iterator_base(), InfoObj() {}
+        : Ptr(Ptr), NumItemsInBucketLeft(0), NumEntriesLeft(NumEntries),
+          InfoObj(InfoObj) {}
+    data_iterator()
+        : Ptr(nullptr), NumItemsInBucketLeft(0), NumEntriesLeft(0),
+          InfoObj(nullptr) {}
+
+    bool operator==(const data_iterator &X) const {
+      return X.NumEntriesLeft == NumEntriesLeft;
+    }
+    bool operator!=(const data_iterator &X) const {
+      return X.NumEntriesLeft != NumEntriesLeft;
+    }
 
     data_iterator &operator++() { // Preincrement
-      this->advance();
+      using namespace llvm::support;
+      if (!NumItemsInBucketLeft) {
+        // 'Items' starts with a 16-bit unsigned integer representing the
+        // number of items in this bucket.
+        NumItemsInBucketLeft =
+            endian::readNext<uint16_t, little, unaligned>(Ptr);
+      }
+      Ptr += sizeof(hash_value_type); // Skip the hash.
+      // Determine the length of the key and the data.
+      const std::pair<offset_type, offset_type> &L =
+          Info::ReadKeyDataLength(Ptr);
+      Ptr += L.first + L.second;
+      assert(NumItemsInBucketLeft);
+      --NumItemsInBucketLeft;
+      assert(NumEntriesLeft);
+      --NumEntriesLeft;
       return *this;
     }
     data_iterator operator++(int) { // Postincrement
-      data_iterator tmp = *this;
-      ++*this;
-      return tmp;
+      data_iterator tmp = *this; ++*this; return tmp;
     }
 
     value_type operator*() const {
-      auto *LocalPtr = this->getItem();
+      const unsigned char *LocalPtr = Ptr;
+      if (!NumItemsInBucketLeft)
+        LocalPtr += 2; // number of items in bucket
+      LocalPtr += sizeof(hash_value_type); // Skip the hash.
 
       // Determine the length of the key and the data.
-      auto L = Info::ReadKeyDataLength(LocalPtr);
+      const std::pair<offset_type, offset_type> &L =
+          Info::ReadKeyDataLength(LocalPtr);
 
       // Read the key.
       const internal_key_type &Key = InfoObj->ReadKey(LocalPtr, L.first);
@@ -601,15 +552,20 @@ public:
   static OnDiskIterableChainedHashTable *
   Create(const unsigned char *Buckets, const unsigned char *const Payload,
          const unsigned char *const Base, const Info &InfoObj = Info()) {
+    using namespace llvm::support;
     assert(Buckets > Base);
-    auto NumBucketsAndEntries =
-        OnDiskIterableChainedHashTable<Info>::readNumBucketsAndEntries(Buckets);
+    assert((reinterpret_cast<uintptr_t>(Buckets) & 0x3) == 0 &&
+           "buckets should be 4-byte aligned.");
+
+    offset_type NumBuckets =
+        endian::readNext<offset_type, little, aligned>(Buckets);
+    offset_type NumEntries =
+        endian::readNext<offset_type, little, aligned>(Buckets);
     return new OnDiskIterableChainedHashTable<Info>(
-        NumBucketsAndEntries.first, NumBucketsAndEntries.second,
-        Buckets, Payload, Base, InfoObj);
+        NumBuckets, NumEntries, Buckets, Payload, Base, InfoObj);
   }
 };
 
 } // end namespace llvm
 
-#endif
+#endif // LLVM_SUPPORT_ON_DISK_HASH_TABLE_H
